"""ss7_to_csv.py

『SS7』データを読み込み、結果csvを出力する関数群です。

Copyright (C) 2024 UNION SYSTEM Inc.

"""
import os
import csv
from Python import Ss7Python as cmd  # type: ignore


def create_main(data_pathlist: tuple[str, str], file_name: str) -> None:
    """create_main

    ss7_to_createのメイン関数です。

    Args:
        pathlist (tuple[str, str]): 物件のパス名のリスト
        file_name (str): 新しく生成する物件名

    """
    print("◆◆◆◆◆作成開始◆◆◆◆◆")
    # csvを出力
    csvpath = (create_csv(data_pathlist[0], 0), create_csv(data_pathlist[1], 1))
    if csvpath and csvpath[0] != "" and csvpath[1] != "":    # csvを編集
        if csv_edit(csvpath):
            create_ss7(csvpath[1], file_name)
    delete_csv_files(csvpath)
    print("*******終了*******")


def create_csv(ikn_path: str, num: int) -> str:
    """create_csv

    『SS7』のパスより入力csvを出力する関数です。

    Args:
        ikn_path (str): 物件のパス名
        num(int):0 = 反映元データ , 1 = 反映先データ
    Returns:
        str: 作成したcsv名

    """
    # 初期化
    cmd.Init()
    cmd.Start("1.1.1.19", 1)  # 1=ログファイルをクリアする

    # データを開く
    # 1=変換して開く 3=物件データは開かない
    Data: cmd.Ss7Data = cmd.Open(ikn_path, 3, 1)

    # データが開けなかった場合
    if (Data is None):
        print("データが開けませんでした")
        return ""

    # 出力するcsvパスを設定
    name = "反映先データ" if num else "反映元データ"
    currentpath: str = os.path.dirname(os.path.abspath(__file__))
    csvpath: str = os.path.join(currentpath, f"{name}.csv")

    # 入力データの取得
    inp = Data.GetInputData()
    # 入力CSVの出力
    # 引数  1 = 同一ファイルが存在する場合、上書きする
    # 　　  1 = 符号名や階名に重複がある場合、続行する
    inp.ExportInputCsv(csvpath, 1, 1)

    # データを閉じる
    Data.Close(2)  # 1=保存しない
    cmd.End(2)  # 1＝保存して終了
    return csvpath


def read_data(file_path: str, section_name: str) -> list:
    """read_data
    指定されたCSVファイルから特定のセクションのデータを読み取ります。

    Args:
        file_path (str): 読み込むCSVファイルのパス
        section_name (str): 取得するセクションの名前（例: "name=特殊荷重登録 節点"）

    Returns:
        list: セクションのデータを含むリスト
    """
    data = []
    found_section = False

    with open(file_path, 'r', encoding='cp932') as f:
        reader = csv.reader(f)
        for row in reader:
            if row and row[0] == section_name:
                found_section = True
                data.append(row)  # セクション名もリストに追加
                continue
            if found_section:
                if row and row[0].startswith("name=") and row[0] != section_name:
                    break
                data.append(row)
    if not found_section:
        print(f"{section_name}が見つかりませんでした。", file_path)
    # 最後の行が空行の場合は削除
    if data and not any(data[-1]):
        data.pop()
    return data


def csv_edit(csvpaths: tuple[str, str]) -> bool:
    """csv_edit
    指定されたCSVファイルを編集し、特定のセクションのデータを追加または修正します。

    Args:
        csvpaths (tuple[str, str]): 編集対象のCSVファイルのパスを含むタプル (元ファイル, 編集ファイル)

    Returns:
        bool : falseの場合は物件を生成しない
    """
    sections = ["name=特殊荷重登録 節点", "name=節点補正重量"]
    new_lines = []

    # 指定されたセクションのデータを読み込む
    data_add = [read_data(csvpaths[1], sections[0]),
                read_data(csvpaths[1], sections[1])]
    data = [read_data(csvpaths[0], sections[0]),
            read_data(csvpaths[0], sections[1])]

    # そもそも反映するデータにセクションが存在しない場合は物件を生成しない
    if not data[0]:
        print("反映元のデータに節点補正重量の指定がありませんでした。")
        return False

    # ファイルを読み込み、既存の内容をリストに保存
    with open(csvpaths[1], 'r', encoding='cp932') as f:
        reader = csv.reader(f)
        for row in reader:
            new_lines.append(row)

    # セクションが存在しない場合は最下行に追加する
    if not data_add[0]:
        for dat in data:
            new_lines.append([])  # 空行を追加
            new_lines.extend(dat)  # リストをそのまま追加
    # セクションが存在する場合は後ろに追加する
    else:
        change = update_registered_data(new_lines, data_add[0], data[0])
        update_pos_data(new_lines, data_add[1], data[1], change)
    # ファイルに書き込む
    with open(csvpaths[1], 'w', newline='', encoding='cp932') as f:
        writer = csv.writer(f)
        writer.writerows(new_lines)  # リストをそのまま書き込み
    return True


def update_registered_data(lines: list[list[str]], data_add: list[list[str]],
                           data: list[list[str]]) -> list[tuple[str, str]]:
    """update_registered_data
    指定されたデータを更新し、変更内容を出力します。

    Args:
        lines (list[list[str]]) : 更新対象のデータ行。
        data_add (list[list[str]]) : 追加するデータ行。
        data (list[list[str]]) : 元のデータ行。

    Returns:
        list[list[str]] : 更新後のデータ行。
    """
    changes: list[tuple[str, str]] = []
    # data_add部分を削除
    if data_add:
        start_index = lines.index(data_add[0])
        end_index = start_index + len(data_add)
        del lines[start_index:end_index]
    # 追加するデータを作成
    count = next((i for i, row in enumerate(data) if '<data>' in row), len(data))
    # data[count + 1:]の要素の1つめを連番に変更
    for i, row in enumerate(data[count + 1:], start=1):
        old_v = row[0]
        row[0] = str(int(data_add[-1][0]) + i)
        changes.append((old_v, row[0]))  # 変更前後の数値を記録
    data_add_new = data_add + data[count + 1:]  # 新しいデータを追加
    lines[start_index:start_index] = data_add_new  # 指定位置に挿入
    return changes


def update_pos_data(lines: list[list[str]], data_add: list[list[str]],
                    data: list[list[str]], changes: list[tuple[str, str]]) -> None:
    """update_pos_data
    この関数は、lines リストに対して data_add を追加し、data リストの内容を changes に基づいて修正します。

    Args:
        lines: 元のデータが格納されたリスト
        data_add: 追加するデータのリスト
        data: 修正するデータのリスト
        changes: 修正内容を示すタプルのリスト (before, after)

    """
    if data_add:
        # data_add部分を削除
        start_index = lines.index(data_add[0])
        end_index = start_index + len(data_add)
        del lines[start_index:end_index]
    # dataを修正
    found_data = False
    for dat in data:
        if found_data:
            dat[3] = next((after for before, after in changes
                           if dat[3] == before), dat[3])
        if '<data>' in dat:
            found_data = True

    if not data_add:
        lines.append([])  # 空行を追加
        lines.extend(data)  # 指定位置に挿入
    else:
        # 追加するデータを作成
        count = next((i for i, row in enumerate(data) if '<data>' in row), len(data))
        # data_add[count + 1:]の要素をチェックして、data[count + 1:]に同じものがあれば変更
        for data_row in data[count + 1:]:
            max_value = max((int(row[2]) for row in data_add[count + 1:]
                             if row[0] == data_row[0] and row[1] == data_row[1]),
                            default=0)
            data_row[2] = str(max_value + int(data_row[2]))
        data_add_new = data_add + data[count + 1:]  # 新しいデータを追加
        lines[start_index:start_index] = data_add_new  # 指定位置に挿入


def delete_csv_files(csvpaths: tuple[str, str]) -> None:
    """
    指定されたCSVファイルが存在する場合に削除する関数

    Args:
        file_paths (list[str]): 削除するCSVファイルのパスのリスト
    """
    for file_path in csvpaths:
        if os.path.exists(file_path) and file_path.endswith('.csv'):
            os.remove(file_path)
            print(f"{file_path} を削除しました。")


def create_ss7(csvpath: str, file_name: str) -> None:
    """create_csv

    入力csvより『SS7』の新規データを作成する関数です。

    Args:
        ikn_path (str): 物件のパス名
        num(int):0 = 反映元データ , 1 = 反映先データ

    """
    # 初期化
    cmd.Init()
    cmd.Start("1.1.1.19", 1)  # 1=ログファイルをクリアする
    # ss7パスの作成
    file_name = file_name + '.ikn' if not file_name.endswith('.ikn') else file_name
    ss7_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), file_name)
    # CSV入力データからSS7のデータを作成する
    cmd.CreateDataCsv(csvpath, ss7_path, 2)  # 2=中断する
    # エラー情報を取得する
    err = cmd.GetLastError()
    # ファイルが既に存在する
    if os.path.isdir(ss7_path):
        print("既に同名の物件データが存在します。", ss7_path)
    # データが開けなかった場合
    elif (err.IsOK() is False):
        print("CSV新規作成が失敗しました")
